{$I-}
{$V-}
{$M 4096,0,0}

program update;

uses dos,crt;

const
  buflen = 4096;
  verbose       : boolean = false;
  adding        : boolean = false;
  dots          : boolean = false;
  dayold        : boolean = false;
  makereadwrite : boolean = false;
  makedestdir   : boolean = false;
  queryfirst    : boolean = false;
  makebackup    : boolean = false;
  testonly      : boolean = false;

var
  dirinfo                 : searchrec;
  srcdir,dstdir           : dirstr;
  srcname,dstname         : namestr;
  srcextn,dstextn         : extstr;
  param,i,numread,attrib  : word;
  thisparam               : string;
  srcfile,dstfile         : file;
  srcfilename,dstfilename : pathstr;
  dstexists,copying       : boolean;
  srctime,dsttime         : longint;
  block                   : array[0..buflen-1] of char;   {used for copying data}
  answer                  : char;

procedure syntaxmessage;
  begin
    writeln('UPDATE: Updates files in one directory from another directory - 22 August 1996.');
    writeln('  by Peter Summers <peter@cardiology.medrmh.unimelb.edu.au>');
    writeln;
    writeln('SYNTAX: UPDATE <source filespec> <destination directory> [/ADD] [/VRB]');
    writeln('               [/DOT] [/DAY] [/MRW] [/MKD] [/QUERY] [/BAK] [/TEST]');
    halt(1)
  end;

begin

  if (paramcount < 2) then syntaxmessage;

  for param := 3 to paramcount do begin
    thisparam := paramstr(param);
    for i := 1 to length(thisparam) do thisparam[i] := upcase(thisparam[i]);
    if thisparam = '/ADD' then
      adding := true
    else if thisparam = '/VRB' then
      verbose := true
    else if thisparam = '/DOT' then
      dots := true
    else if thisparam = '/DAY' then
      dayold := true
    else if thisparam = '/MRW' then
      makereadwrite := true
    else if thisparam = '/MKD' then
      makedestdir := true
    else if thisparam = '/QUERY' then
      queryfirst := true
    else if thisparam = '/BAK' then
      makebackup := true
    else if thisparam = '/TEST' then
      testonly := true
    else syntaxmessage;
  end;

  fsplit(paramstr(1),srcdir,srcname,srcextn);
  dstdir := paramstr(2);

  if not (dstdir[length(dstdir)] in ['\','/',':']) then begin
    assign(dstfile,dstdir);
    getfattr(dstfile,attrib);
    if (doserror<>0) or ((attrib and directory) = 0) then begin
      writeln('Directory '+paramstr(2)+' does not exist.');
      if makedestdir then begin
        mkdir(dstdir);
        if IOResult=0 then
          writeln('Directory '+paramstr(2)+' created.')
        else begin
          writeln('Can''t create directory '+paramstr(2)+'.');
          halt(1);
        end
      end else
        halt(1);
    end;
    dstdir := dstdir + '\';
  end;

  FileMode := 0;   { Source files are open Read-Only. }

  write('Updating '+paramstr(1)+' to '+paramstr(2)+'...');

  findfirst(paramstr(1),0,dirinfo);
  if doserror=0 then repeat

    srcfilename := srcdir + dirinfo.name;
    srctime := dirinfo.time;
    dstfilename := dstdir + dirinfo.name;

    assign(dstfile,dstfilename);
    reset(dstfile,1);
    dstexists := (IOResult=0);

    if dstexists then begin
      getftime(dstfile,dsttime);
      if dayold then
        copying := ((dsttime and $FFFF0000) <= (srctime and $FFFF0000))
      else
        copying := (dsttime < srctime);
      close(dstfile);
      if IOResult<>0 then begin
        write(^M+^J+'Can''t close file '+dstfilename);
        copying := false;
      end;
    end else
      copying := adding;

    if copying and queryfirst then repeat
      write(^M+^J+'Update '+srcfilename+' to '+dstfilename+ ' ? ');
      answer := upcase(readkey);
      if answer = 'N' then copying := false;
      if answer =  ^[ then halt;
      write(answer);
    until answer in ['N','Y',^[];

    if copying and testonly then begin
      write(^M+^J+srcfilename+' would be copied to '+dstfilename);
      copying := false;
    end;

    if copying then begin
      assign(srcfile,srcfilename);
      reset(srcfile,1);
      if (IOresult<>0) then begin
        write(^M+^J+'Can''t open file '+srcfilename);
        copying := false;
      end;
    end;

    if copying and dstexists and makereadwrite then begin
      setfattr(dstfile,0);
      if (IOresult<>0) then begin
        write(^M+^J+'Can''t make writeable file '+dstfilename);
        close(srcfile);
        if IOResult<>0 then write(^M+^J+'Can''t close file '+srcfilename);
        copying := false;
      end;
    end;

    if copying and dstexists and makebackup then begin
      fsplit(dstfilename,dstdir,dstname,dstextn);
      rename(dstfile,dstdir+'\'+dstname+'.BAK');
      assign(dstfile,dstfilename);
      if (IOresult<>0) then begin
        write(^M+^J+'Can''t rename file '+dstfilename);
        close(srcfile);
        if IOResult<>0 then write(^M+^J+'Can''t close file '+srcfilename);
        copying := false;
      end;
    end;

    if copying then begin
      rewrite(dstfile,1);
      if (IOresult<>0) then begin
        write(^M+^J+'Can''t open file '+dstfilename);
        close(srcfile);
        if IOResult<>0 then write(^M+^J+'Can''t close file '+srcfilename);
        copying := false;
      end;
    end;

    while copying and not eof(srcfile) do begin
      blockread(srcfile,block[0],sizeof(block),numread);
      if (IOresult<>0) then begin
        write(^M+^J+'Can''t read from file '+srcfilename);
        close(srcfile);
        if IOResult<>0 then {ignore it};
        close(dstfile);
        if IOresult<>0 then write(^M+^J+'Can''t close file '+dstfilename);
        copying := false;
      end else begin
        blockwrite(dstfile,block[0],numread);
        if (IOresult<>0) then begin
          write(^M+^J+'Can''t write to file '+dstfilename);
          close(srcfile);
          if IOResult<>0 then write(^M+^J+'Can''t close file '+srcfilename);
          close(dstfile);
          if IOResult<>0 then {ignore it};
          copying := false;
        end;
      end;
    end;

    if copying then begin
      if not dayold then setftime(dstfile,srctime);
      close(dstfile);
      if IOResult<>0 then
        write(^M+^J+'Can''t close file '+dstfilename)
      else
        write(^M+^J+srcfilename+' copied to '+dstfilename);
      close(srcfile);
      if IOResult<>0 then
        write(^M+^J+'Can''t close file '+srcfilename);
    end else {if not copying then} begin
      if verbose then
        write(^M+^J+srcfilename+' not copied to ',dstfilename);
      if dots then if dstexists then write('*') else write('.');
    end;

    findnext(dirinfo);

  until doserror<>0 else

    write(^M+^J+'No files found matching '+paramstr(1));

  writeln;

end.

